import logging
import os
import asyncio
import aiohttp
import re
from flask import Flask, jsonify, render_template, request
from bs4 import BeautifulSoup
from datetime import datetime

app = Flask(__name__)

# Configuração de logging
logging.basicConfig(level=logging.INFO)

# Caminho para exportação dos resultados
RESULTS_DIR = '/home/jaali/public_html/resultados/coleta1'
TIMESTAMP = datetime.now().strftime("%Y%m%d_%H%M%S")
EXPORT_PATH = RESULTS_DIR

# Cria a pasta para armazenar os resultados
os.makedirs(EXPORT_PATH, exist_ok=True)

progresso = []  # Lista para armazenar o progresso

# Função para buscar email, WhatsApp, nome e endereço da URL da rádio
async def buscar_contatos(session, url_radio):
    try:
        logging.info(f'Acessando rádio: {url_radio}')
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36',
            'Referer': 'https://www.google.com'
        }
        async with session.get(url_radio, headers=headers, timeout=30) as response:
            response.raise_for_status()
            soup = BeautifulSoup(await response.text(), 'html.parser')

            contatos = {'whatsapp': None, 'email': None, 'nome': None, 'endereço': None}

            nome_element = soup.find('div', class_='info').find('h1')
            endereço_element = soup.find('div', class_='info').find('h2')
            if nome_element:
                contatos['nome'] = nome_element.get_text(strip=True)
            if endereço_element:
                contatos['endereço'] = endereço_element.get_text(strip=True)

            for link in soup.find_all('a', href=True):
                href = link['href']
                if 'wa.me' in href or 'api.whatsapp.com' in href or 'whatsapp.com/send' in href:
                    contatos['whatsapp'] = href.split('=')[-1]
                    break

            email_link = soup.find('a', href=lambda x: x and x.startswith('mailto:'))
            if email_link:
                contatos['email'] = email_link['href'].replace('mailto:', '')
            else:
                for element in soup.find_all(text=True):
                    email_text = element.strip()
                    if '@' in email_text:
                        match = re.search(r'[\w\.-]+@[\w\.-]+', email_text)
                        if match:
                            contatos['email'] = match.group(0)
                            break

                if not contatos['email']:
                    for tag in soup.find_all(attrs={"data-value": True}):
                        data_value = tag['data-value']
                        if '@' in data_value:
                            contatos['email'] = data_value
                            break

            return contatos['email'], contatos['whatsapp'], contatos['nome'], contatos['endereço']
    except Exception as e:
        logging.error(f'Erro ao acessar {url_radio}: {str(e)}')
        return None, None, None, None

# Função para exportar resultados para CSV
def exportar_resultados_contato(email, nome, endereço):
    try:
        if email:  # Somente adiciona se o email estiver presente
            file_path = os.path.join(EXPORT_PATH, 'resultados.csv')
            with open(file_path, 'a', encoding='utf-8') as file:
                linha = f"{email},{nome or ''},{endereço or ''}\n"
                file.write(linha)
    except Exception as e:
        logging.error(f'Erro ao exportar resultados: {str(e)}')

# Função para exportar WhatsApp para um arquivo separado
def exportar_resultados_whatsapp(whatsapp):
    try:
        if whatsapp:
            file_path = os.path.join(EXPORT_PATH, 'whatsapp.txt')
            with open(file_path, 'a', encoding='utf-8') as file:
                file.write(f"{whatsapp}\n")
    except Exception as e:
        logging.error(f'Erro ao exportar WhatsApp: {str(e)}')

# Função para acessar uma página e coletar URLs das rádios
async def coletar_urls_radio(session, page_number):
    url_pagina = f"https://www.radios.com.br/radio/pais/brasil/33?pg={page_number}"
    try:
        logging.info(f'Acessando página: {url_pagina}')
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36',
            'Referer': 'https://www.google.com'
        }
        async with session.get(url_pagina, headers=headers) as response:
            response.raise_for_status()
            soup = BeautifulSoup(await response.text(), 'html.parser')

            urls_radios = []
            for radio_item in soup.select('.resultado a.btn-primary'):
                radio_url = radio_item['href']
                urls_radios.append(radio_url)

            return urls_radios
    except Exception as e:
        logging.error(f'Erro ao acessar página {page_number}: {str(e)}')
        return []

# Função principal para iterar por todas as páginas e coletar URLs das rádios
async def coletar_todas_as_urls():
    async with aiohttp.ClientSession() as session:
        for page_number in range(1, 1271):
            urls_de_pagina = await coletar_urls_radio(session, page_number)
            for url_radio in urls_de_pagina:
                email, whatsapp, nome, endereço = await buscar_contatos(session, url_radio)
                exportar_resultados_contato(email, nome, endereço)
                exportar_resultados_whatsapp(whatsapp)
                progresso.append(f'Capturando {page_number}/{1270} - {url_radio}')
                await asyncio.sleep(5)

            await asyncio.sleep(1)

    logging.info("Todas as URLs foram coletadas e salvas.")

# Endpoint para obter progresso
@app.route('/progresso/')
def progresso_contato():
    return jsonify(progresso)

# Página inicial para iniciar a captura
@app.route('/iniciar-captura/', methods=['GET', 'POST'])
async def iniciar_captura():
    global progresso
    if request.method == 'POST':
        progresso = []
        await coletar_todas_as_urls()
        progresso.append("FIM DE CAPTURA")
        logging.info("FIM DE CAPTURA")
        return jsonify(progresso)
    return '''
    <!DOCTYPE html>
    <html lang="pt-BR">
    <head>
        <meta charset="UTF-8">
        <title>Iniciar Captura</title>
        <script src="https://code.jquery.com/jquery-3.6.0.min.js"></script>
        <script>
            $(document).ready(function() {
                $('#captureForm').on('submit', function(event) {
                    event.preventDefault();
                    $('#progress').text('Iniciando captura...');

                    $.ajax({
                        url: '/iniciar-captura/',
                        type: 'POST',
                        success: function(response) {
                            $('#progress').html('<h3>Captura concluída!</h3>' + response.join('<br>'));
                        },
                        error: function(xhr) {
                            $('#progress').text('Erro: ' + xhr.responseText);
                        }
                    });

                    const atualizarProgresso = setInterval(function() {
                        $.get('/progresso/', function(data) {
                            $('#progress').append('<br>' + data.join('<br>'));
                        });
                    }, 2000);
                });
            });
        </script>
    </head>
    <body>
        <h1>Iniciar Captura de Contatos</h1>
        <form id="captureForm">
            <button type="submit">Iniciar</button>
        </form>
        <div id="progress"></div>
    </body>
    </html>
    '''

# Executar o aplicativo Flask
if __name__ == '__main__':
    app.run(host='0.0.0.0', port=5001)
